@@ -1,20 +1,27 @@
 # Step 02
 # This code is used to merge imputed files from TopMed based on list of saved variants in prior steps
 # The goal is to combine post-imputation vcf.gz files into one vcf.gz file
-import pandas as pd
+
 from xopen import PipedCompressionWriter, xopen # Faster than gzip
 import os
 # import process_args
-from .process_args import process_args
-from .get_SNP_list import get_snp_list
+# from .process_args import process_args
+# from .get_SNP_list import get_snp_list
 # from .check_r2_setting_for_imputation import check_imputatilson_parameters
 import time
 
+if __name__ == "__main__":
+    from process_args import process_args
+    from get_SNP_list import get_snp_list
+else: # import this way when used as a module. '.' indicates to import from the current module (directory)
+    from .process_args import process_args
+    from .get_SNP_list import get_snp_list
+
 # ################################ Helper functions ################################
 # Print progress par in console
 # - progress: current progress (number of SNPs processed)
 def progress_bar(progres):
-    total = number_snps_kept # total number of SNPs need to be precessed
+    total = number_snps_kept # total number of SNPs needs to be processed
     percent = 100 * (progres/total)
     bar = '=' * int(percent) + '-' * int(100 - percent)
     print(f'|{bar}| {percent:.2f}%', end='\r')
@@ -22,7 +29,6 @@ def progress_bar(progres):
 
 # This function prints execution time at the end of run
 def print_execution_time(start_time):
-    # merge_files()
     duration = time.time() - start_time
     hours = duration // 3600
     duration = duration % 3600
@@ -119,7 +125,7 @@ def merge_header_lines(lst_input_fh, fh_output):
     write_args(fh_output)
 
     # Merge column headers
-    if line[0] != '#': # Sanity check
+    if line[0] != '#': # Sanity check, can also use: assert line[0] == '#', 'header line should start with #'
         print('Error: header line should start with #\nExit')
         LOG_TXT += 'Error: header line should start with #\nExit\n'
         exit()
@@ -283,14 +289,37 @@ def merge_files(dict_flags, inx_info_column, inx_indiv_id_starts, lst_input_fh,
 # ################################ End of helper functions ################################
 
 
-# A function to run each step all together
-# Params:
-# - args: a dictionary contains arguments when IMMerged is used as a python module
-def run_merge_files(args=''):
+
+def run_merge_files(args):
+    '''
+    A function to run merging steps
+    Params:
+    - args: a dictionary containing arguments when IMMerged is used as a python module, formatted as {'--input':['file1', 'file2'], '--missing':1}. Valid flags are:
+        --input: (Required) Files to be merged, multiple files are allowed. Must in gzipped or bgziped VCF format.
+        --info: (Optional) Directory/name to info files.
+                Default path is the same directory as corresponding input file, default info files share the same name as input file, except for suffix (.info.gz).
+        --output: (Optional) Default is "merged.vcf.gz" and saved at current working directory. Output file name without suffix.
+        --thread: (Optional) Default value is 1. Defines how many thread to use in multiprocessing. If number of threads <0, will use 1 instead of user supplied value.
+        --missing: (Optional) Default is 0. Defines number of missing values allowed for each variant.
+        --na_rep: (Optional) Default is "." (ie. ".|." for genotype values). Defines what symbol to use for missing values. This flag is ignored if --missing is 0.
+        --r2_threshold: (Optional) Default is 0, ie. no filtering. Only variants with combined imputation quality score r2≥r2_threshold will be saved in the merged file.
+        --r2_output: (Optional) Default is "z_transformation". Defines how imputation quality score is calculated in the output file.
+        --r2_cap: (Optional) Default is 0.001. Adjust R squared by --r2_cap if imputation quality Rsq=1. Only valid for z transformation to avoid infinity.
+        --duplicate_id: (Optional) Default is 0. Defines number of duplicate individuals in each input file. Duplicated IDs should be the first N columns in each file.
+        --check_duplicate_id: (Optional) Default is False. Check if there are duplicate IDs, then rename non-first IDs to ID:2, ID:3, ..., ID:index_of_input_file+1.
+        --write_with: (Optional) Default is bgzip. Write to bgziped file with bgzip. User can supply specific path to bgzip such as /user/bin/bgzip.
+        --meta_info: (Optional) Valid values are {index of input file (1-based), 'none', 'all'}.
+                    What meta information (lines start with '##') to include in output file. Default is 1 (meta information from the first input file).
+        --use_rsid: (Optional) Default is False. If input VCFs use rsID instead of chr:pos:ref:alt, set this option to True to avoid duplicate IDs (rsID may not be unique).
+                    Use make_info.py to make info files, or follow the required format.
+        --verbose: (Optional) Default is False. Print more messages.
+    Return:
+    - Save a merged and bgziped vcf file as specified in --output
+    '''
     start_time = time.time()  # Track execution time
 
     global VERSION  # Global variable Version of IMMerge
-    VERSION = '0.0.2'
+    VERSION = '0.0.3'
     global LOG_TXT
     LOG_TXT = ''  # Global variable to track info printed. Write to log file when run is finished
 
@@ -302,21 +331,17 @@ def run_merge_files(args=''):
     # Process terminal input or arguments passed through python module call
     # dict_flags contains values for IMMerge flags:
     global dict_flags
-    # Sanity check: check if args is empty when used as python module
-    if __name__ != '__main__':
-        if args == '': # Missing arguments
-            print('Error: args argument in run_merge_files() is empty. Must pass arguments in a dictionary when IMMerge is called as python module \nExit')
-            print_execution_time(start_time)
-            exit()
-        else: # Parse arguments, convert to a list and pass to process_args()
-            args_list = []
-            for k, v in args.items():
-                args_list.append(k)
-                if isinstance(v, list): # add all values if it is a list
-                    for val in v: args_list.append(val)
-                else: args_list.append(str(v)) # Args from command line are strings, so need to convert for process_args() to works
-            dict_flags = process_args(args_list)
-    else: # If called from terminal
+
+    if args != '': # If used as a module
+        # Parse arguments, convert to a list and pass to process_args()
+        args_list = []
+        for k, v in args.items():
+            args_list.append(k)
+            if isinstance(v, list): # add all values if it is a list
+                for val in v: args_list.append(val)
+            else: args_list.append(str(v)) # Args from command line are strings, so need to convert for process_args() to works
+        dict_flags = process_args(args_list)
+    else: # If called from terminal, so args==''
         dict_flags = process_args(args)  # Process terminal input, use it as a global variable
 
     for k, v in dict_flags.items():
@@ -365,5 +390,5 @@ def run_merge_files(args=''):
     log_fh.write(LOG_TXT+'\n')
 
 if __name__ == '__main__': # Called as commandline tool
-    run_merge_files()
-# else: # If run as a module in python script, must pass a list of arguments to run_merge_files()
+    run_merge_files('')
+# If run as a module in python script, must pass a dictionary arguments to run_merge_files()
